"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@actions/core");
const github_1 = require("@actions/github");
const github_action_helper_1 = require("@technote-space/github-action-helper");
exports.Logger = github_action_helper_1.Logger;
exports.ContextHelper = github_action_helper_1.ContextHelper;
exports.Command = github_action_helper_1.Command;
exports.ApiHelper = github_action_helper_1.ApiHelper;
exports.GitHelper = github_action_helper_1.GitHelper;
exports.Utils = github_action_helper_1.Utils;
const misc_1 = require("./utils/misc");
exports.isTargetContext = misc_1.isTargetContext;
const process_1 = require("./utils/process");
exports.execute = process_1.execute;
exports.showActionInfo = github_action_helper_1.ContextHelper.showActionInfo;
exports.getLogger = (logger) => (logger !== null && logger !== void 0 ? logger : new github_action_helper_1.Logger());
const getActionContext = (option) => ({
    actionContext: github_1.context,
    actionDetail: option,
});
/**
 * @param {object} option option
 * @param {string} option.actionName action name
 * @param {string} option.actionOwner action owner
 * @param {string} option.actionRepo action repo
 * @param {Logger|undefined} option.logger logger
 * @param {string|undefined} option.message message
 * @return {Promise<void>} void
 */
function main(option) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        if (option.rootDir) {
            exports.showActionInfo(option.rootDir, exports.getLogger(option.logger), github_1.context);
        }
        if (!misc_1.isTargetContext(getActionContext(option))) {
            exports.getLogger(option.logger).info((_a = option.notTargetEventMessage, (_a !== null && _a !== void 0 ? _a : 'This is not target event.')));
            return;
        }
        yield process_1.execute(getActionContext(option));
    });
}
exports.main = main;
/**
 * @param {object} option option
 * @param {Logger|undefined} option.logger logger
 * @param {string|undefined} option.message message
 */
function run(option) {
    main(option).catch(error => core_1.setFailed(error.message));
}
exports.run = run;
//# sourceMappingURL=index.js.map